/*
 * Copyright (C) 2005
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

/*
	<<		op1 << op2		Shift bits of op1 left by distance op2; fills with zero bits on the right-hand side 					<br>
	>>		op1 >> op2		Shift bits of op1 right by distance op2; fills with highest (sign) bit on the left-hand side		<br>
	>>>		op1 >>> op2		Shift bits of op1 right by distance op2; fills with zero bits on the left-hand side					<br>
*/
package estella.lib;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.util.StringTokenizer;

/**
* Descriptor Payloads (little-endian)
* @author Daniel Machado
* @since 21/11/2005
*/
public class Payload
{
  private ByteArrayOutputStream bytes;
  private DataOutputStream PONG, QUERY, QUERYHITS;
	
	/**
	* Payload PING  0x00
	* @return PING - byte zerado
	*/
	public byte[] ping()
	{
		byte[] PING = new byte[1];
		PING[0] = 0x00000000;
		return PING;
	}
		
	/**
	* Payload PONG	0x01
	* Estrutura:
	* 	|porta|host|n_arq_comp|tam_arq_comp| <br>
	*	|--|-----|-----|-----|     <br>  |2|4|4|4|  (bytes)
	* Todos inteiros em little-endian
	* @param porta	 inteiro com a porta 
	* @param host  String com o host
	* @param n_arq_comp  inteiro com o numero de arquivos compartilhados
	* @param tam_arq_comp  inteiro com o tamanho total desses arquivos compartilhados
	* @return byte[]  vetor com os bytes do payload
	*/
	public byte[] pong(int porta, String host, int n_arq_comp, int tam_arq_comp)
	{
		try {
			bytes = new ByteArrayOutputStream();
			PONG = new DataOutputStream(bytes);
			
			// Porta (0~1)
			PONG.write(0x00FF & porta);					// byte 0
			PONG.write((0xFF00 & porta) >> 8);	// byte 1
			
			// Host (2~5) - endereo de ip - big-endian
			StringTokenizer thost = new StringTokenizer(host, ".");
			PONG.write(Integer.parseInt(thost.nextToken()));	// byte 2
			PONG.write(Integer.parseInt(thost.nextToken()));	// byte 3
			PONG.write(Integer.parseInt(thost.nextToken()));	// byte 4
			PONG.write(Integer.parseInt(thost.nextToken()));	// byte 5
			
			// Numero de arquivos compartilhados (6-9)
			PONG.write(0x000000FF & n_arq_comp);					// byte 6
			PONG.write((0x0000FF00 & n_arq_comp) >> 8);		// byte 7
			PONG.write((0x00FF0000 & n_arq_comp) >> 16);	// byte 8
			PONG.write((0xFF000000 & n_arq_comp) >> 24);	// byte 9
			
			// Tamanho total dos arquivos compartilhados (10-13)
			PONG.write(0x000000FF & tam_arq_comp);					// byte 10
			PONG.write((0x0000FF00 & tam_arq_comp) >> 8);		// byte 11
			PONG.write((0x00FF0000 & tam_arq_comp) >> 16);	// byte 12
			PONG.write((0xFF000000 & tam_arq_comp) >> 24);	// byte 13
			
			// Retorna o payload do tipo pong (vetor de bytes)
			return(bytes.toByteArray());
		}
		catch(Exception FalhaGerarPong)
		{
			return null;
		}
	}
	
	/**
	* Payload Query  0x080
	* @param min_velocidade inteiro com a velocidade minima
	* @param busca String com a sentenca de caracteres a ser buscada
	* @return query - array de bytes com o payload do tipo query
	*/
	public byte[] query(int min_velocidade, String busca)
	{
		try {
			int i=0;
			
			// Array QUERY
			byte[] QUERY = new byte[busca.length() + 3];
			
			// Array de bytes com a string de busca
			byte[] bbusca = busca.getBytes();
			
			// Velocidade minima (0~1)
			QUERY[i] = (byte)(0x00FF & min_velocidade);						// byte 1 
			QUERY[++i] = (byte)((0xFF00 & min_velocidade) >> 8);	// byte 2
			
			// String de busca (2~N) aonde N = tamanho da string busca
			for(;i < busca.length(); i++)
				QUERY[i] = bbusca[i];
			
			// Finaliza (N~N+1)
			QUERY[++i] = (byte) 0x00;
			
			// Retorna o payload do tipo pong (vetor de bytes)
			return(QUERY);
		}
		catch(Exception FalhaGerarQuery)
		{
			return null;
		}
	}
	
	/**
	* Payload QueryHits  (0x81)
	* @param 
	*
	*/
	public byte[] queryhits(byte hits, int porta, String host, byte[] resultset)
	{
		try {
			int i=0, x=0;
			byte[] queryhits = new byte[11 + resultset.length];
			
			// Numero de hits (0)
			queryhits[i] =   (byte)(0x00FF & hits);		// byte 0
			
			// Porta (1~2)
			queryhits[++i] = (byte)(0x00FF & porta);	// byte 1
			queryhits[++i] = (byte)(0xFF00 & porta);	// byte 2

			// Host (2~5) - endereo de ip - big-endian
			StringTokenizer thost = new StringTokenizer(host, ".");
			queryhits[++i] = (byte) Integer.parseInt(thost.nextToken());	// byte 3
			queryhits[++i] = (byte) Integer.parseInt(thost.nextToken());	// byte 4
			queryhits[++i] = (byte) Integer.parseInt(thost.nextToken());	// byte 5
			queryhits[++i] = (byte) Integer.parseInt(thost.nextToken());	// byte 6
			
			// Velocidade maxima de upload (nao funciona ~ desabilitado)
			queryhits[++i] = (byte) 0x00000000;		// byte 7
			queryhits[++i] = (byte) 0x00000000;		// byte 8
			queryhits[++i] = (byte) 0x00000000;		// byte 9
			queryhits[++i] = (byte) 0x00000000;		// byte 10
			
			// Result Set - conjunto de resultados (11~N)
			for(; i < resultset.length; x++)
				queryhits[++i] = (byte) (resultset[x]);
			return queryhits;
		}
		catch(Exception FalhaQueryHits)
		{
			return null;
		}
	}
	
	public byte[] hit(int index, int tam_arquivo, String nome)
	{
		byte[] bnome = nome.getBytes();
		byte[] hit = new byte[9+bnome.length];
		int i=0, x=0;
		
		// File index (0~3)
		hit[i] = (byte) (0x000000FF & index);							// byte 0
		hit[++i] = (byte) ((0x0000FF00 & index) >> 8);		// byte 1
		hit[++i] = (byte) ((0x00FF0000 & index) >> 16);		// byte 2
		hit[++i] = (byte) ((0xFF000000 & index) >> 24);		// byte 3
		
		// Tamanho do arquivo (4~7)
		hit[i] = (byte) (0x000000FF & tam_arquivo);							// byte 4
		hit[++i] = (byte) ((0x0000FF00 & tam_arquivo) >> 8);		// byte 5
		hit[++i] = (byte) ((0x00FF0000 & tam_arquivo) >> 16);		// byte 6
		hit[++i] = (byte) ((0xFF000000 & tam_arquivo) >> 24);		// byte 7
		
		// Nome do arquivo (8~tamanho do nome do arquivo)
		for(; i < bnome.length; x++)
			hit[++i] = bnome[x];
		
		hit[++i] = (byte) 0x0000;
		return hit;
	}
	
	public byte[] concatenaHit(byte[] hit1, byte[] hit2)
	{
		int i, j;
		byte[] hit3 = new byte[hit1.length + hit2.length];
		
		for(i=0; i<hit1.length; i++)
			hit3[i] = hit1[i];
		for(; i<hit2.length; i++)
			hit3[i] = hit2[i];
		
		return hit3;
	}
}
